--[[
The MIT License (MIT)

Copyright (c) 2015 Ludvig Alexander Verpe

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
]]--

local TB = {}

TB.textBoxes = {}
TB.inputPool = {}
TB.matchPattern = '^[%g%s]$'
TB.backspaceTimer= 0
TB.blinkingTimer = 0
TB.typingCursor = "|"
TB.backspaceTimerLimit= 0.12
TB.backspaceHold = 0
TB.blinkingTimerLimit= 0.6
TB.pause = false

function TB.setButton(n, b, t, f)
	for i,v in ipairs(TB.textBoxes) do
		if v.name == n then
			if b == 1 then
				v.button1 = {t, f}
			end
			if b == 2 then
				v.button2 = {t, f}
			end
			if b == 3 then
				v.button3 = {t, f}
			end
		end
	end
end

function TB.setBorder(n, s)
	for i,v in ipairs(TB.textBoxes) do
		if v.name == n then
			v.b = s
		end
	end
end

function TB.unselect()
	for i,v in ipairs(TB.textBoxes) do
		if v.selected then
			v.selected = false
		end
	end
end

function TB.getTextSelected()
	for i,v in ipairs(TB.textBoxes) do
		if v.selected then
			return v.text
		end
	end
end

function TB.getNameSelected()
	for i,v in ipairs(TB.textBoxes) do
		if v.selected then
			return v.name
		end
	end
end

function TB.setSelectable(n,s)
	for i,v in ipairs(TB.textBoxes) do
		if v.name == n then
			v.selectable = t
		end
	end
end

function TB.getSelectable(n)
	for i,v in ipairs(TB.textBoxes) do
		if v.name == n then
			return v.selectable
		end
	end
end

function TB.setText(n,t)
	for i,v in ipairs(TB.textBoxes) do
		if v.name == n then
			v.text = t
		end
	end
end

function TB.getText(n)
	for i,v in ipairs(TB.textBoxes) do
		if v.name == n then
			return v.text
		end
	end
end

function TB.setTypingCursor(c)
	TBtypingCursor = c
end

function TB.setBackspaceTime(t)
	TBbackspaceTimerLimit = t
end

function TB.setBlinkTime(t)
	TBblinkingTimerLimit = t
end

function TB.setClearText(n, c)
	for i,v in ipairs(TB.textBoxes) do
		if v.name == n then
			v.clt = c
		end
	end
end

function TB.getClearText(n, c)
	for i,v in ipairs(TB.textBoxes) do
		if v.name == n then
			return v.clt
		end
	end
end

function TB.setDraw(n, d)
	for i,v in ipairs(TB.textBoxes) do
		if v.name == n then
			v.draw = d
		end
	end
end

function TB.getDraw(n)
	for i,v in ipairs(TB.textBoxes) do
		if v.name == n then
			return v.draw
		end
	end
end

function TB.paused(p)
	TB.pause = p
end

function TB.setDrawAll(d)
	TBdrawAll = d
end

function TB.getDrawAll()
	return TBdrawAll
end

function TB.destroyBox(n)
	for i,v in ipairs(TB.textBoxes) do
		if v.name == n then
			table.remove(TB.textBoxes, i)
		end
	end
end

function TB.destroyAll()
	for i = 1,#TB.textBoxes do
		table.remove(TB.textBoxes, 1)
	end
end

function TB.rectColl( x1, y1, w1, h1, x2, y2, w2, h2 )
	if x1 + w1 > x2 and x1 < x2 + w2 and y1 + h1 > y2 and y1 < y2 + h2 then		
		return true			
	else
		return false	
	end
end

function TB.create(X, Y, W, H, B, T, F, TOX, TOY, C1, C2, C6, C3, C4, C5, N, D, CLT, CENTER, DBG, SLCT, BT1, BT2, BT3)
	if not W then W = 150 end
	if not H then H = 18 end
	if not B then B = 1 end
	if not T then T = "" end
	if not F then F = love.graphics.newFont(12) end
	if not TOX then TOX = 3 end
	if not TOY then TOY = 3 end
	if not C1 then C1 = {0, 0, 0} end
	if not C2 then C2 = {220, 220, 220} end
	if not C3 then C3 = {170, 170, 170} end
	if not C4 then C4 = {0, 0, 0} end
	if not C5 then C5 = {30, 30, 30} end
	if not C6 then C6 = {255, 255, 255} end
	if not N then N = "box" .. #TB.textBoxes + 1 end
	if D == nil then D = true end
	if CLT == nil then CLT = false end
	if CENTER == nil then CENTER = false end
	if DBG == nil then DBG = true end
	if SLCT == nil then SLCT = true end
	if not BT1 then BT1 = false end
	if not BT2 then BT2 = false end
	if not BT3 then BT3 = false end
	table.insert(TB.textBoxes, {
		x = X, 
		y = Y, 
		w = W, 
		h = H, 
		b = B, 
		color1 = C1,
		color2 = C2,
		color3 = C3,
		color4 = C4,
		color5 = C5,
		color6 = C6,
		text = T,
		font = F,
		tox = TOX,
		toy = TOY,
		name = N,
		selected = false,
		draw = D,
		clt = CLT,
		center = CENTER,
		dbg = DBG,
		selectable = SLCT,
		button1 = BT1,
		button2 = BT2,
		button3 = BT3,
		bt1selected = false,
		bt2selected = false,
		bt3selected = false,
	})
end

function TB.draw()
	if not TB.pause then
		for i,v in ipairs(TB.textBoxes) do
			if v.draw then
				if v.selected and v.selectable then
					love.graphics.setColor(v.color3)
				else
					love.graphics.setColor(v.color1)
				end
			
				if v.b > 0 and v.w > 0 then 
					love.graphics.rectangle("line",v.x, v.y, v.w, v.h)
				end
				
				if v.dbg and v.selected and v.w > 0 then
					if v.selectable then
						love.graphics.setColor(v.color6)
					else
						love.graphics.setColor(v.color2)
					end
					love.graphics.rectangle("fill",v.x + v.b, v.y + v.b, v.w - (v.b * 2), v.h - (v.b * 2))
				elseif v.dbg and not v.selected and v.w > 0 then
					love.graphics.setColor(v.color2)
					love.graphics.rectangle("fill",v.x + v.b, v.y + v.b, v.w - (v.b * 2), v.h - (v.b * 2))
				end
		
				love.graphics.setFont(v.font)
				
				if v.w > 0 then
					if v.center then
						if v.selected and v.selectable and love.keyboard.isDown("lctrl") and love.keyboard.isDown("n") then
							love.graphics.setColor(v.color5)
							love.graphics.print(v.name, v.x + (v.w / 2) - (v.font:getWidth(v.name) / 2) , v.y + v.toy)
						elseif v.selected and v.selectable and TB.blinkingTimer < TB.blinkingTimerLimit/2 then
							love.graphics.setColor(v.color5)
							love.graphics.print(v.text .. TB.typingCursor, v.x + (v.w / 2) - (v.font:getWidth(v.text) / 2) , v.y + v.toy)
						elseif not v.selected or not v.selectable then
							love.graphics.setColor(v.color4)
							love.graphics.print(v.text, v.x + (v.w / 2) - (v.font:getWidth(v.text) / 2) , v.y + v.toy)
						else
							love.graphics.setColor(v.color5)
							love.graphics.print(v.text, v.x + (v.w / 2) - (v.font:getWidth(v.text) / 2) , v.y + v.toy)
						end
					else
						if v.selected and v.selectable and love.keyboard.isDown("lctrl") and love.keyboard.isDown("n") then
							love.graphics.setColor(v.color5)
							love.graphics.print(v.name, v.x + v.tox , v.y + v.toy)
						elseif v.selected and v.selectable and TB.blinkingTimer < TB.blinkingTimerLimit/2 then
							love.graphics.setColor(v.color5)
							love.graphics.print(v.text .. TB.typingCursor, v.x + v.tox , v.y + v.toy)
						elseif not v.selected or not v.selectable then
							love.graphics.setColor(v.color4)
							love.graphics.print(v.text, v.x + v.tox , v.y + v.toy)
						else
							love.graphics.setColor(v.color5)
							love.graphics.print(v.text, v.x + v.tox , v.y + v.toy)
						end
					end
				end
				if v.button1 then
					if v.bt1selected and v.selectable then
						love.graphics.setColor(v.color3)
					else
						love.graphics.setColor(v.color1)
					end
					
					if v.b > 0 then 
						love.graphics.rectangle("line",v.x + v.w + 3, v.y, v.font:getWidth(v.button1[1]) + (v.tox*2), v.h)
					end
					
					if v.dbg and v.bt1selected then
						if v.selectable then
							love.graphics.setColor(v.color6)
						else
							love.graphics.setColor(v.color2)
						end
						love.graphics.rectangle("fill",v.x + v.w + 3 + v.b, v.y + v.b, v.font:getWidth(v.button1[1]) + (v.tox*2) - (v.b * 2), v.h - (v.b * 2))
					elseif v.dbg and not v.bt1selected  then
						love.graphics.setColor(v.color2)
						love.graphics.rectangle("fill",v.x + v.w + 3 + v.b, v.y + v.b, v.font:getWidth(v.button1[1]) + (v.tox*2) - (v.b * 2), v.h - (v.b * 2))
					end
					
					if v.bt1selected and v.selectable then
						love.graphics.setColor(v.color5)
					else
						love.graphics.setColor(v.color4)
					end
					love.graphics.print(v.button1[1], v.x + v.w + 3 + v.tox , v.y + v.toy)
				end
				
				if v.button2 then
					if v.bt2selected and v.selectable then
						love.graphics.setColor(v.color3)
					else
						love.graphics.setColor(v.color1)
					end
					
					if v.b > 0 then 
						love.graphics.rectangle("line",v.x + v.w + 6 + v.font:getWidth(v.button1[1]) + (v.tox*2), v.y, v.font:getWidth(v.button2[1]) + (v.tox*2), v.h)
					end
					
					if v.dbg and v.bt2selected then
						if v.selectable then
							love.graphics.setColor(v.color6)
						else
							love.graphics.setColor(v.color2)
						end
						love.graphics.rectangle("fill",v.x + v.w + 6 + v.b + v.font:getWidth(v.button1[1]) + (v.tox*2), v.y + v.b, v.font:getWidth(v.button2[1]) + (v.tox*2) - (v.b * 2), v.h - (v.b * 2))
					elseif v.dbg and not v.bt2selected  then
						love.graphics.setColor(v.color2)
						love.graphics.rectangle("fill",v.x + v.w + 6 + v.b + v.font:getWidth(v.button1[1]) + (v.tox*2), v.y + v.b, v.font:getWidth(v.button2[1]) + (v.tox*2) - (v.b * 2), v.h - (v.b * 2))
					end
					
					if v.bt2selected and v.selectable then
						love.graphics.setColor(v.color5)
					else
						love.graphics.setColor(v.color4)
					end
					love.graphics.print(v.button2[1], v.x + v.w + 6 + v.font:getWidth(v.button1[1]) + (v.tox*3) , v.y + v.toy)
				end
				
				if v.button3 then
					if v.bt3selected and v.selectable then
						love.graphics.setColor(v.color3)
					else
						love.graphics.setColor(v.color1)
					end
					
					if v.b > 0 then 
						love.graphics.rectangle("line",v.x + v.w + 9 + v.font:getWidth(v.button1[1]) + v.font:getWidth(v.button2[1]) + (v.tox*4), v.y, v.font:getWidth(v.button3[1]) + (v.tox*2), v.h)
					end
					
					if v.dbg and v.bt3selected then
						if v.selectable then
							love.graphics.setColor(v.color6)
						else
							love.graphics.setColor(v.color2)
						end
						love.graphics.rectangle("fill",v.x + v.w + 9 + v.b + v.font:getWidth(v.button1[1]) + v.font:getWidth(v.button2[1]) + (v.tox*4), v.y + v.b, v.font:getWidth(v.button3[1]) + (v.tox*2) - (v.b * 2), v.h - (v.b * 2))
					elseif v.dbg and not v.bt3selected  then
						love.graphics.setColor(v.color2)
						love.graphics.rectangle("fill",v.x + v.w + 9 + v.b + v.font:getWidth(v.button1[1]) + v.font:getWidth(v.button2[1]) + (v.tox*4), v.y + v.b, v.font:getWidth(v.button3[1]) + (v.tox*2) - (v.b * 2), v.h - (v.b * 2))
					end
					
					if v.bt3selected and v.selectable then
						love.graphics.setColor(v.color5)
					else
						love.graphics.setColor(v.color4)
					end
					love.graphics.print(v.button3[1], v.x + v.w + 9 + (v.b*2) + (v.tox * 1.5) + v.font:getWidth(v.button1[1]) + v.font:getWidth(v.button2[1]) + (v.tox*3) , v.y + v.toy)
				end
			end
		end
	end
end

function TB.update(dt)
	if not TB.pause then
		if TB.backspaceTimer < TB.backspaceTimerLimit then 
			TB.backspaceTimer = TB.backspaceTimer+ dt 
		end
	
		if TB.blinkingTimer < TB.blinkingTimerLimit then 
			TB.blinkingTimer = TB.blinkingTimer + dt
		else
			TB.blinkingTimer = 0
		end
	
		for i,v in ipairs(TB.textBoxes) do
			if TB.rectColl(v.x, v.y, v.w, v.h, love.mouse.getX(), love.mouse.getY(), 0, 0) and love.mouse.isDown("l") and v.w > 0 then
				v.selected = true
				if v.clt then
					v.text = ""
				end
			elseif not TB.rectColl(v.x, v.y, v.w, v.h, love.mouse.getX(), love.mouse.getY(), 0, 0) and love.mouse.isDown("l") then
				v.selected = false
				love.keyboard.setTextInput(false)
			end
			
			if v.button1 and TB.rectColl(v.x + v.w + 3, v.y, v.font:getWidth(v.button1[1]) + (v.tox*2), v.h, love.mouse.getX(), love.mouse.getY(), 0, 0) and love.mouse.isDown("l") then
				v.selected = true
				v.bt1selected = true
				v.button1[2]()
			else
				v.bt1selected = false
			end
			
			if v.button2 and TB.rectColl(v.x + v.w + 6 + v.font:getWidth(v.button1[1]) + (v.tox*2), v.y, v.font:getWidth(v.button2[1]) + (v.tox*2), v.h, love.mouse.getX(), love.mouse.getY(), 0, 0) and love.mouse.isDown("l") then
				v.selected = true
				v.bt2selected = true
				v.button2[2]()
			else
				v.bt2selected = false
			end
			
			if v.button3 and TB.rectColl(v.x + v.w + 9 + v.font:getWidth(v.button1[1]) + v.font:getWidth(v.button2[1]) + (v.tox*4), v.y, v.font:getWidth(v.button3[1]) + (v.tox*2), v.h, love.mouse.getX(), love.mouse.getY(), 0, 0) and love.mouse.isDown("l") then
				v.selected = true
				v.bt3selected = true
				v.button3[2]()
			else
				v.bt3selected = false
			end
		
			if v.selected then
				if love.keyboard.isDown("return") then
					v.selected = false
					love.keyboard.setTextInput(false)
				else
					love.keyboard.setTextInput(true)
				end
			
				if love.keyboard.isDown("backspace") and TB.backspaceTimer > TB.backspaceTimerLimit - TB.backspaceHold then
					TB.backspaceTimer = 0
					TB.backspaceHold = TB.backspaceHold + (dt/2)
					if TB.backspaceHold >= TB.backspaceTimerLimit - 0.01 then
						TB.backspaceHold = TB.backspaceTimerLimit - 0.01
					end
					v.text = string.sub(v.text, 1, -2)
					
				elseif not love.keyboard.isDown("backspace") then
					TB.backspaceHold = 0
				end
			
				if love.keyboard.isDown("lctrl") and love.keyboard.isDown("v") and TB.backspaceTimer > TB.backspaceTimerLimit and v.font:getWidth(v.text .. love.system.getClipboardText()) < v.w - (v.tox * 6) then
					TB.backspaceTimer= 0
					v.text = v.text .. love.system.getClipboardText()
				end
	
				function love.textinput(t)
					if t:match( TB.matchPattern ) and v.font:getWidth(v.text) < v.w - (v.tox * 4) and v.selected then
						v.text = v.text .. t
					end
				end
			end
		
			if v.selected and love.keyboard.isDown("lctrl") and love.keyboard.isDown("c") then
				love.system.setClipboardText(v.text)
			end
		end
	end
end
return TB